#ifndef W1_Interface
#define   W1_Interface        0                         // Nur 1 Pin
#endif
//#define   W1_Interface        1                         // 1 Pin als negierter Ausgang und 1 Pin als Eingang
//#define   W1_Interface        2                         // Advanced Interface von Dallas/Maxim

#define   W1_BitRD            W1_Bit
#if W1_Interface==0
    #define   W1_BitWR            W1_Bit
  #else
    #ifndef W1_BitWR
    #define   W1_BitWR            P3_2
    #endif
#endif

#if (W1_Interface==2)
    #define   W1_BitDrive         P3_2
    #define   W1_BitDPU           P3_4
#endif

// Functions-Prototypen
void InitW1(void);
unsigned char W1_Reset(void);
unsigned char W1_Read_Bit(void);
void W1_Write_Bit(unsigned char WBit);
unsigned char W1_Read_Byte(void);
void W1_Write_Byte(unsigned char WByte);
unsigned char W1_ROM_Search(unsigned char W1_Buffer[7]);
int ReadTemperatur(void);

void Delay_2us(void);
void Delay_8us(void);
void Delay_15us(void);
void Delay_16us(void);
void Delay_18us(void);
void Delay_45us(void);
void Delay_60us(void);
void Delay_66us(void);
void Delay_72us(void);
void Delay_100us(void);
void Delay_300us(void);
void Delay_480us(void);

// 1-Wire ROM-Befehle
#define   Match_ROM           0x55
#define   Skip_ROM            0xCC
#define   Search_ROM          0xF0
#define   Read_ROM            0x33

#define   Alarm_Search        0xEC

// 1-Wire Funktions-Befehle
#define   Convert             0x44
#define   Write_Scratch       0x4E
#define   Read_Scratch        0xBE
#define   Copy_Scratch        0x48
#define   Recall              0xB8
#define   Read_Power          0xB4

// 1-Wire Status
#define   Search_First        0xFF
#define   Presence_ERR        0xFF
#define   Data_ERR            0xFF
#define   Last_Device         0x00

#define   ROMCodeDS18S20      0x10
#define   ROMCodeDS18B20      0x28
#define   ROMCodeDS2405       0x05
#define   ROMCodeDS2406       0x12

// DS1820-befehele
#define   Read_Scratchpad     0xBE


#include "1Wire.c"


// X=12000 / t(s)
#ifndef DelayA
  #define   DelayA              4                         // Zyklen fr Aufruf
#endif
#define   Wait15              (Xtal+799)/800*X2Set/2      //  15 * xtal / 12000
#define   Wait60              (Xtal+199)/200*X2Set/2      //  60 * xtal / 12000
#define   Wait480             (Xtal+ 24)/ 25*X2Set/2      // 480 * xtal / 12000
#define   Wait300             (Xtal+ 39)/ 40*X2Set/2      // 300 * xtal / 12000
#define   Wait100             (Xtal+119)/120*X2Set/2      // 100 * xtal / 12000
#define   Wait66              (Xtal+181)/182*X2Set/2      //  66 * xtal / 12000
#define   Wait45              (Xtal+266)/267*X2Set/2      //  45 * xtal / 12000

#define   Wait2               (Xtal+5999)/6000*X2Set/2    //   2 * xtal / 12000
#define   Wait8               (Xtal+1499)/1500*X2Set/2    //   8 * xtal / 12000
#define   Wait16              (Xtal+ 749)/ 750*X2Set/2    //  16 * xtal / 12000
#define   Wait18              (Xtal+ 666)/ 667*X2Set/2    //  18 * xtal / 12000
#define   Wait72              (Xtal+ 166)/ 167*X2Set/2    //  72 * xtal / 12000

void Delay_2us(void)
  {
    _asm
         #if (Wait2<DelayA)
          #else
             MOV    DPL,#Wait2-DelayA
loop2:       DJNZ   DPL,loop2
         #endif
    _endasm;
  }

void Delay_8us(void)
  {
    _asm
         #if (Wait8<DelayA)
          #else
             MOV    DPL,#Wait8-DelayA
loop8:       DJNZ   DPL,loop8
         #endif
    _endasm;
  }

void Delay_15us(void)
  {
    _asm
         #if (Wait15<DelayA)
          #else
             MOV    DPL,#Wait15-DelayA
loop15:      DJNZ   DPL,loop15
         #endif
    _endasm;
  }

void Delay_16us(void)
  {
    _asm
         #if (Wait16<DelayA)
          #else
             MOV    DPL,#Wait16-DelayA
loop16:      DJNZ   DPL,loop16
         #endif
    _endasm;
  }

void Delay_18us(void)
  {
    _asm
         #if (Wait18<DelayA)
          #else
             MOV    DPL,#Wait18-DelayA
loop18:      DJNZ   DPL,loop18
         #endif
    _endasm;
  }

void Delay_45us(void)
  {
    _asm
         #if (Wait45<DelayA)
          #else
             MOV    DPL,#Wait45-DelayA
loop45:      DJNZ   DPL,loop45
         #endif
    _endasm;
  }

void Delay_60us(void)
  {
    _asm
         #if (Wait60<DelayA)
          #else
             MOV    DPL,#Wait60-DelayA
loop60:      DJNZ   DPL,loop60
         #endif
    _endasm;
  }

void Delay_66us(void)
  {
    _asm
         #if (Wait66<DelayA)
          #else
             MOV    DPL,#Wait66-DelayA
loop66:      DJNZ   DPL,loop66
         #endif
    _endasm;
  }

void Delay_72us(void)
  {
    _asm
         #if (Wait72<DelayA)
          #else
             MOV    DPL,#Wait72-DelayA
loop72:      DJNZ   DPL,loop72
         #endif
    _endasm;
  }

void Delay_100us(void)
  {
    _asm
         #if (Wait100<DelayA)
          #else
           #if (Wait100>255)
             MOV    DPL,#(Wait100/2)-DelayA
loop100:     NOP
             NOP
             DJNZ   DPL,loop100
            #else
             MOV    DPL,#Wait100-DelayA
loop100:     DJNZ   DPL,loop100
           #endif
         #endif
    _endasm;
  }

void Delay_300us(void)
  {
    _asm
         #if (Wait300<DelayA)
          #else
           #if (Wait300>255)
             MOV    DPL,#(Wait300/2)-DelayA
loop300:     NOP
             NOP
             DJNZ   DPL,loop300
            #else
             MOV    DPL,#Wait300-DelayA
loop300:     DJNZ   DPL,loop300
           #endif
         #endif
    _endasm;
  }

void Delay_480us(void)
  {
    _asm
         #if (Wait480<DelayA)
          #else
           #if (Wait480>255)
             MOV    DPL,#(Wait480/2)-DelayA
loop480:     NOP
             NOP
             DJNZ   DPL,loop480
            #else
             MOV    DPL,#Wait480-DelayA
loop480:     DJNZ   DPL,loop480
           #endif
         #endif
    _endasm;
  }
