// Nokia Display
//#define CS_Disp     P1_4
//#define DC          P1_5                // 0=>Command; 1=>Data

//#define DisplayType3410 1



#if DisplayType3410
    #define MaxXPos     96
    #define MaxYPos     65
  #else
    // Nokia 3310
    #define MaxXPos     84
    #define MaxYPos     48
#endif

#define TabWidth        8
#define EOF             '\0'

unsigned char XPos, YPos;
bit invers=0;

//*****************************************************************************
// Nokia 3310/3410 Routinen
void InitDisplay(void);
void ClearScreen(void);
void SetCursor(unsigned char XSoll, unsigned char YSoll);
int puts(char *s);
void putchar(char c);
unsigned char GetFontData(unsigned char *z, unsigned char Zeichen);

void InitStdOut(void)
  {
    CS_Disp = 1;
    DC = 0;

    CS_Disp = 0;
    SendSPI(0x21);           // Function set; Power ON, Horizontal Adressing, Ext. Instruction Set

#ifdef EEPROM_Adr
    EECON = 2;                       // EEPROM einschalten
                  MOV       DPTR,#EEPROM_Adr+Vop
                  MOVX      A,@DPTR
                  ORL       A,#10000000b
    EECON = 0                        // EEPROM ausschalten
  #else
    SendSPI(0xBE);           // Vop set
#endif
    SendSPI(0x06);           // Temp. Coeff.
#ifdef DisplayType3410
    SendSPI(0x14);           // BIAS Mode 1:48
  #else
    SendSPI(0x12);           // BIAS Mode 1:48
#endif
    SendSPI(0x0A);           // HV-gen voltage multiplication
    SendSPI(0x20);           // Standart Command, Horiz. Adr.
    SendSPI(0x11);           // VLCD range select
    SendSPI(0x0C);           // LCD in Normal Mode
//    SendSPI(0x09);           // LCD alle Segmente ein
//    SendSPI(0x08);           // LCD alle Segmente aus
//    SendSPI(0x0D);           // LCD in Invers Mode
    CS_Disp = 1;
    DC = 1;

    ClearScreen();
  }

void ClearScreen(void)
  {
    unsigned char x, y;

    SetCursor(0, 0);
    CS_Disp = 0;
    for (x=0;x<MaxXPos;x++)
      {
        for (y=0;y<MaxYPos;y++)
          {
            SendSPI(0);
          }
      }
    CS_Disp = 1;
    SetCursor(0, 0);
  }

void SetCursor(unsigned char XSoll, unsigned char YSoll)
   {
      DC = 0;
      CS_Disp = 0;
      SendSPI(0x20);
      SendSPI(0x40+YSoll);
      SendSPI(0x80+XSoll);
      CS_Disp = 1;
      DC = 1;

      XPos = XSoll;
      YPos = YSoll;
    }
    
void SetContrast(unsigned char Value)
   {
      DC = 0;
      CS_Disp = 0;
      SendSPI(0x21);                  // Function set; Power ON, Horizontal Adressing, Ext. Instruction Set
      SendSPI(Value & 0xBF | 0xA0);
      SendSPI(0x20);                  // Function set; Power ON, Horizontal Adressing, Norm. Instruction Set
      CS_Disp = 1;
      DC = 1;
    }

void putchar(char c)
  {
    unsigned char i;
    unsigned char Anzahl;
    data unsigned char buffer[8];

    switch (c)
      {
        case 0x08: // TAB
             for (i=0; i<TabWidth; i++)
               {
                 putchar(' ');
               }
             break;

        case 0x0A: // Line Feed
             SetCursor(XPos,YPos+1);
             break;

        case 0x0D: // Carige Return
             SetCursor(0, YPos);
             break;

        default:   // Zeichen in Bytes umwandeln und senden
             Anzahl=GetFontData(&buffer,c);

             if (XPos+Anzahl<MaxXPos)
               {
                  // Zeichen passt noch in die Zeile => XPos einfach erhhen
                  XPos=XPos+Anzahl;
               }
             else
               if (XPos+Anzahl>MaxXPos+1)
                 {
                   // Zeichen passt nicht mehr in die Zeile => Cursor an den Anfang der nchsten Zeile setzen
                   SetCursor(0,YPos+1);
                   XPos=Anzahl;
                 }
               else
                 {
                   // Zeichen passt noch genau in die Zeile => XPos auf 0 setzen und YPos um eins erhhen
                   XPos=0;
                   YPos++;
                   if (XPos+Anzahl==MaxXPos+1)
                     {
                       Anzahl--;
                     }
                 }

             CS_Disp = 0;
             for (i=0;i<Anzahl;i++)
               {
                 if (invers)
                   {
                     SendSPI(~buffer[i]);
                   }
                  else  
                   {
                     SendSPI(buffer[i]);
                   }
               }
             CS_Disp = 1;

             SetCursor(XPos,YPos);
      }
  }

void WriteByte(unsigned char Value)
  {
    CS_Disp = 0;
    SendSPI(Value);
    CS_Disp = 1;
    XPos=XPos+1;
  }  

unsigned char GetFontData(unsigned char *z, unsigned char Zeichen)
  {
    //*z wird in DPTR bergeben
    DPH = Zeichen;

    _asm

//;******************************************************************************
//;* Ld die Fontdaten, fr das ASCII Zeichen in R0, in DisplayBuffer und       *
//;* die Anzahl der Bytes zurck                                                *
//;*                                                                            *
//;* Input:  R0                                                                 *
//;* Output: DispBuffer, Anzahl                                                 *
//;* Used:                                                                      *
//;* Stack:  19                                                                 *
//;******************************************************************************
                    PUSH      ACC
                    MOV       A,R0
                    PUSH      ACC
                    MOV       A,R1
                    PUSH      ACC
                    MOV       A,R3
                    PUSH      ACC
                    MOV       A,R4
                    PUSH      ACC
                    MOV       A,R5
                    PUSH      ACC

                    MOV       R1,DPL                        ; DispBuffer setzen
                    MOV       R0,DPH                        ; Zeichen setzen

                    CLR       A                             ; ASCII_Zeichen - Letztes_Zeichen
                    MOV       DPTR,#LastChar
                    MOVC      A,@A+DPTR
                    MOV       R3,A
                    MOV       A,R0
                    SETB      C                             ; Damit das Carry-Bit auch beim letzten Zeichen in Font gesetzt wird
                    SUBB      A,R3
                    JNC       NoFontData                    ; Zeichen ist nicht definiert

                    CLR       A                             ; ASCII_Zeichen - Erstes_Zeichen
                    MOV       DPTR,#FirstChar
                    MOVC      A,@A+DPTR
                    MOV       R3,A
                    MOV       A,R0
                    CLR       C
                    SUBB      A,R3
                    JC        NoFontData                    ; Zeichen ist nicht definiert

                    MOV       R3,A
                    MOV       DPTR,#FontCharData
                    MOV       R4,DPL
                    MOV       R5,DPH
                    MOV       DPTR,#FontCharSize

                    MOV       A,R3
                    JZ        ReadFirstCharData

CalcBytePos:        CLR       A                             ; Zeichen holen
                    MOVC      A,@A+DPTR

                    ADD       A,R4                          ; Anzahl zum Offset addieren
                    MOV       R4,A
                    CLR       A
                    ADDC      A,R5
                    MOV       R5,A

                    INC       DPTR                          ; Zeiger auf nchstes Zeichen
                    DJNZ      R3,CalcBytePos

ReadFirstCharData:  CLR       A                             ; Anzahl Bytes des bentigten Zeichens holen
                    MOVC      A,@A+DPTR

                    MOV       R3,A
                    MOV       R0,A   ; Anzahl ersetzt

                    MOV       DPL,R4
                    MOV       DPH,R5

                    lJMP      ReadFontCharData

NoFontData:         MOV       R0,#0x00   ; Anzahl ersetzt

                    lJMP      EndFontCharData

ReadFontCharData:   CLR       A
                    MOVC      A,@A+DPTR
//                    JNB       Invers,ReadFontCharData1
//                    CPL       A
ReadFontCharData1:  MOV       @R1,A

                    INC       DPTR
                    INC       R1
                    DJNZ      R3,ReadFontCharData

                    CLR       A                             ; eine Spalte Platz zwischen den Zeichen lassen
//                    JNB       Invers,NoSetInvers
//                    CPL       A
NoSetInvers:        MOV       @R1,A
                    INC       R0   ; Anzahl ersetzt

EndFontCharData:    MOV       DPL,R0   ; Anzahl ersetzt                    ; Parameter Anzahl zurckgeben

                    POP       ACC
                    MOV       R5,A
                    POP       ACC
                    MOV       R4,A
                    POP       ACC
                    MOV       R3,A
                    POP       ACC
                    MOV       R1,A
                    POP       ACC
                    MOV       R0,A
                    POP       ACC

                    lJMP      Ende

Font8x8:
FontSize:           .DW 0x01B6
FontHeight:         .DB 0x08
FontWidth:          .DB 0x08
BitperPixel:        .DB 0x01
FirstChar:          .DB 0x20
LastChar:           .DB 0x7A

FontCharSize:       .DB 0x01, 0x01, 0x03, 0x05, 0x04, 0x07, 0x05, 0x02, 0x02, 0x02, 0x03, 0x03, 0x01, 0x03, 0x01, 0x04
                    .DB 0x04, 0x02, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x04, 0x01, 0x01, 0x03, 0x03, 0x03, 0x04
                    .DB 0x06, 0x06, 0x05, 0x05, 0x05, 0x04, 0x04, 0x05, 0x05, 0x01, 0x04, 0x05, 0x04, 0x07, 0x06, 0x05
                    .DB 0x04, 0x05, 0x04, 0x04, 0x05, 0x05, 0x05, 0x07, 0x05, 0x05, 0x05, 0x02, 0x04, 0x02, 0x05, 0x03
                    .DB 0x02, 0x04, 0x04, 0x04, 0x04, 0x04, 0x02, 0x04, 0x04, 0x01, 0x01, 0x04, 0x01, 0x07, 0x04, 0x05
                    .DB 0x04, 0x04, 0x02, 0x04, 0x02, 0x04, 0x04, 0x05, 0x03, 0x03, 0x03

FontCharData:       .DB 0x00, 0x5F, 0x07, 0x00, 0x07, 0x24, 0x7E, 0x24, 0x7E, 0x24, 0x24, 0x7A, 0x2F, 0x12, 0x42, 0x25
                    .DB 0x12, 0x08, 0x24, 0x52, 0x21, 0x30, 0x4E, 0x59, 0x36, 0x28, 0x00, 0x07, 0x3E, 0x41, 0x41, 0x3E
                    .DB 0x28, 0x1C, 0x28, 0x08, 0x1C, 0x08, 0xC0, 0x08, 0x08, 0x08, 0x40, 0x40, 0x30, 0x0C, 0x03, 0x3E
                    .DB 0x41, 0x41, 0x3E, 0x02, 0x7F, 0x62, 0x51, 0x49, 0x46, 0x22, 0x41, 0x49, 0x36, 0x18, 0x16, 0x7F
                    .DB 0x10, 0x2F, 0x45, 0x45, 0x39, 0x3E, 0x49, 0x49, 0x32, 0x01, 0x71, 0x0D, 0x03, 0x36, 0x49, 0x49
                    .DB 0x36, 0x26, 0x49, 0x49, 0x3E, 0x48, 0xC8, 0x08, 0x14, 0x22, 0x14, 0x14, 0x14, 0x22, 0x14, 0x08
                    .DB 0x02, 0x51, 0x09, 0x06, 0x3E, 0x41, 0x5D, 0x5D, 0x51, 0x1E, 0x60, 0x1C, 0x13, 0x13, 0x1C, 0x60
                    .DB 0x7F, 0x49, 0x49, 0x49, 0x36, 0x3E, 0x41, 0x41, 0x41, 0x22, 0x7F, 0x41, 0x41, 0x41, 0x3E, 0x7F
                    .DB 0x49, 0x49, 0x41, 0x7F, 0x09, 0x09, 0x01, 0x3E, 0x41, 0x49, 0x29, 0x7A, 0x7F, 0x08, 0x08, 0x08
                    .DB 0x7F, 0x7F, 0x30, 0x40, 0x40, 0x3F, 0x7F, 0x08, 0x14, 0x22, 0x41, 0x7F, 0x40, 0x40, 0x40, 0x7F
                    .DB 0x03, 0x0C, 0x30, 0x0C, 0x03, 0x7F, 0x7F, 0x03, 0x0C, 0x30, 0x40, 0x7F, 0x3E, 0x41, 0x41, 0x41
                    .DB 0x3E, 0x7F, 0x09, 0x09, 0x06, 0x3E, 0x41, 0x51, 0x61, 0xBE, 0x7F, 0x09, 0x09, 0x76, 0x26, 0x49
                    .DB 0x49, 0x32, 0x01, 0x01, 0x7F, 0x01, 0x01, 0x3F, 0x40, 0x40, 0x40, 0x3F, 0x03, 0x1C, 0x60, 0x1C
                    .DB 0x03, 0x03, 0x1C, 0x60, 0x18, 0x60, 0x1C, 0x03, 0x63, 0x14, 0x08, 0x14, 0x63, 0x03, 0x04, 0x78
                    .DB 0x04, 0x03, 0x61, 0x51, 0x49, 0x45, 0x43, 0x7F, 0x41, 0x03, 0x0C, 0x30, 0x40, 0x41, 0x7F, 0x04
                    .DB 0x02, 0x01, 0x02, 0x04, 0x40, 0x40, 0x40, 0x03, 0x04, 0x20, 0x54, 0x54, 0x78, 0x7F, 0x44, 0x44
                    .DB 0x38, 0x38, 0x44, 0x44, 0x28, 0x38, 0x44, 0x44, 0x7F, 0x38, 0x54, 0x54, 0x18, 0x7E, 0x09, 0x98
                    .DB 0xA4, 0xA4, 0x7C, 0x7F, 0x08, 0x04, 0x78, 0x7D, 0xFD, 0x7F, 0x18, 0x24, 0x40, 0x7F, 0x7C, 0x04
                    .DB 0x04, 0x7C, 0x04, 0x04, 0x78, 0x7C, 0x04, 0x04, 0x78, 0x38, 0x44, 0x44, 0x44, 0x38, 0xFC, 0x24
                    .DB 0x24, 0x18, 0x18, 0x24, 0x24, 0xFC, 0x7C, 0x04, 0x48, 0x54, 0x54, 0x24, 0x3E, 0x44, 0x3C, 0x40
                    .DB 0x40, 0x7C, 0x1C, 0x60, 0x60, 0x1C, 0x1C, 0x60, 0x18, 0x60, 0x1C, 0x6C, 0x10, 0x6C, 0x9C, 0xA0
                    .DB 0x7C, 0x64, 0x54, 0x4C

Ende:

    _endasm;

    return DPL;
  }

// Ende Nokia 3310/3410 Routinen
//*****************************************************************************
