// ENC28J60 Control Registers

// buffer boundaries applied to internal 8K ram
//	entire available packet buffer space is allocated
#define TXSTART_INIT   	0x0000	// start TX buffer at 0
#define RXSTART_INIT   	0x0600	// give TX buffer space for one full ethernet frame (~1500 bytes)
#define RXSTOP_INIT    	0x1FFF	// receive buffer gets the rest

#define	MAX_FRAMELEN	1518	// maximum ethernet frame length


// MAC address for this interface
#define ENC28J60_MAC0 0x01
#define ENC28J60_MAC1 0x23
#define ENC28J60_MAC2 0x45
#define ENC28J60_MAC3 0x67
#define ENC28J60_MAC4 0x89
#define ENC28J60_MAC5 0xAB


// X=12000 / t(s)
#define   Wait50              (Xtal+240)/241*X2Set/2      //  50 * xtal / 12000
void Delay_50us(void)
  {
    _asm
         #if (Wait50<DelayA)
          #else
             MOV    DPL,#Wait50-DelayA
loop50:      DJNZ   DPL,loop50
         #endif
    _endasm;
  }


// Control register definitions are a combination of address, bank number, and Ethernet/MAC/PHY indicator bits.

// - Register address	(bits 0-4)
// - Bank number		(bits 5-6)
// - MAC/PHY indicator	(bit 7)
#define ADDR_Mask	0x1F
#define Bank_Mask	0x60
#define SPRD_Mask	0x80   // ??? Pascal Stang  9/22/2005

#define Bank0_Mask  0x00
#define Bank1_Mask  0x20
#define Bank2_Mask  0x40
#define Bank3_Mask  0x60

// All-bank registers
#define EIE			0x1B
#define EIR			0x1C
#define ESTAT		0x1D
#define ECON2		0x1E
#define ECON1		0x1F

// Bank 0 registers
#define ERDPTL		(0x00|Bank0_Mask)
#define ERDPTH		(0x01|Bank0_Mask)
#define EWRPTL		(0x02|Bank0_Mask)
#define EWRPTH		(0x03|Bank0_Mask)
#define ETXSTL		(0x04|Bank0_Mask)
#define ETXSTH		(0x05|Bank0_Mask)
#define ETXNDL		(0x06|Bank0_Mask)
#define ETXNDH		(0x07|Bank0_Mask)
#define ERXSTL		(0x08|Bank0_Mask)
#define ERXSTH		(0x09|Bank0_Mask)
#define ERXNDL		(0x0A|Bank0_Mask)
#define ERXNDH		(0x0B|Bank0_Mask)
#define ERXRDPTL	(0x0C|Bank0_Mask)
#define ERXRDPTH	(0x0D|Bank0_Mask)
#define ERXWRPTL	(0x0E|Bank0_Mask)
#define ERXWRPTH	(0x0F|Bank0_Mask)
#define EDMASTL		(0x10|Bank0_Mask)
#define EDMASTH		(0x11|Bank0_Mask)
#define EDMANDL		(0x12|Bank0_Mask)
#define EDMANDH		(0x13|Bank0_Mask)
#define EDMADSTL	(0x14|Bank0_Mask)
#define EDMADSTH	(0x15|Bank0_Mask)
#define EDMACSL		(0x16|Bank0_Mask)
#define EDMACSH		(0x17|Bank0_Mask)

// Bank 1 registers
#define EHT0		(0x00|Bank1_Mask)
#define EHT1		(0x01|Bank1_Mask)
#define EHT2		(0x02|Bank1_Mask)
#define EHT3		(0x03|Bank1_Mask)
#define EHT4		(0x04|Bank1_Mask)
#define EHT5		(0x05|Bank1_Mask)
#define EHT6		(0x06|Bank1_Mask)
#define EHT7		(0x07|Bank1_Mask)
#define EPMM0		(0x08|Bank1_Mask)
#define EPMM1		(0x09|Bank1_Mask)
#define EPMM2		(0x0A|Bank1_Mask)
#define EPMM3		(0x0B|Bank1_Mask)
#define EPMM4		(0x0C|Bank1_Mask)
#define EPMM5		(0x0D|Bank1_Mask)
#define EPMM6		(0x0E|Bank1_Mask)
#define EPMM7		(0x0F|Bank1_Mask)
#define EPMCSL		(0x10|Bank1_Mask)
#define EPMCSH		(0x11|Bank1_Mask)
#define EPMOL		(0x14|Bank1_Mask)
#define EPMOH		(0x15|Bank1_Mask)
#define EWOLIE		(0x16|Bank1_Mask)
#define EWOLIR		(0x17|Bank1_Mask)
#define ERXFCON		(0x18|Bank1_Mask)
#define EPKTCNT		(0x19|Bank1_Mask)

// Bank 2 registers
#define MACON1		(0x00|Bank2_Mask|0x80)
#define MACON2		(0x01|Bank2_Mask|0x80)
#define MACON3		(0x02|Bank2_Mask|0x80)
#define MACON4		(0x03|Bank2_Mask|0x80)
#define MABBIPG		(0x04|Bank2_Mask|0x80)
#define MAIPGL		(0x06|Bank2_Mask|0x80)
#define MAIPGH		(0x07|Bank2_Mask|0x80)
#define MACLCON1	(0x08|Bank2_Mask|0x80)
#define MACLCON2	(0x09|Bank2_Mask|0x80)
#define MAMXFLL		(0x0A|Bank2_Mask|0x80)
#define MAMXFLH		(0x0B|Bank2_Mask|0x80)
#define MAPHSUP		(0x0D|Bank2_Mask|0x80)
#define MICON		(0x11|Bank2_Mask|0x80)
#define MICMD		(0x12|Bank2_Mask|0x80)
#define MIREGADR	(0x14|Bank2_Mask|0x80)
#define MIWRL		(0x16|Bank2_Mask|0x80)
#define MIWRH		(0x17|Bank2_Mask|0x80)
#define MIRDL		(0x18|Bank2_Mask|0x80)
#define MIRDH		(0x19|Bank2_Mask|0x80)

// Bank 3 registers
#define MAADR1		(0x00|Bank3_Mask|0x80)
#define MAADR0		(0x01|Bank3_Mask|0x80)
#define MAADR3		(0x02|Bank3_Mask|0x80)
#define MAADR2		(0x03|Bank3_Mask|0x80)
#define MAADR5		(0x04|Bank3_Mask|0x80)
#define MAADR4		(0x05|Bank3_Mask|0x80)
#define EBSTSD		(0x06|Bank3_Mask)
#define EBSTCON		(0x07|Bank3_Mask)
#define EBSTCSL		(0x08|Bank3_Mask)
#define EBSTCSH		(0x09|Bank3_Mask)
#define MISTAT		(0x0A|Bank3_Mask|0x80)
#define EREVID		(0x12|Bank3_Mask)
#define ECOCON		(0x15|Bank3_Mask)
#define EFLOCON		(0x17|Bank3_Mask)
#define EPAUSL		(0x18|Bank3_Mask)
#define EPAUSH		(0x19|Bank3_Mask)

// PHY registers
#define PHCON1		0x00
#define PHSTAT1		0x01
#define PHID1		0x02
#define PHID2		0x03
#define PHCON2		0x10
#define PHSTAT2		0x11
#define PHIE		0x12
#define PHIR		0x13
#define PHLCON		0x14

// ENC28J60 EIE Register Bit Definitions
#define EIE_INTIE		0x80
#define EIE_PKTIE		0x40
#define EIE_DMAIE		0x20
#define EIE_LINKIE		0x10
#define EIE_TXIE		0x08
#define EIE_WOLIE		0x04
#define EIE_TXERIE		0x02
#define EIE_RXERIE		0x01

// ENC28J60 EIR Register Bit Definitions
#define EIR_PKTIF		0x40
#define EIR_DMAIF		0x20
#define EIR_LINKIF		0x10
#define EIR_TXIF		0x08
#define EIR_WOLIF		0x04
#define EIR_TXERIF		0x02
#define EIR_RXERIF		0x01

// ENC28J60 ESTAT Register Bit Definitions
#define ESTAT_INT		0x80
#define ESTAT_LATECOL	0x10
#define ESTAT_RXBUSY	0x04
#define ESTAT_TXABRT	0x02
#define ESTAT_CLKRDY	0x01

// ENC28J60 ECON2 Register Bit Definitions
#define ECON2_AUTOINC	0x80
#define ECON2_PKTDEC	0x40
#define ECON2_PWRSV		0x20
#define ECON2_VRPS		0x08

// ENC28J60 ECON1 Register Bit Definitions
#define ECON1_TXRST		0x80
#define	ECON1_RXRST		0x40
#define ECON1_DMAST		0x20
#define ECON1_CSUMEN	0x10
#define ECON1_TXRTS		0x08
#define	ECON1_RXEN		0x04
#define ECON1_BSEL1		0x02
#define ECON1_BSEL0		0x01

// ENC28J60 MACON1 Register Bit Definitions
#define MACON1_LOOPBK	0x10
#define MACON1_TXPAUS	0x08
#define MACON1_RXPAUS	0x04
#define MACON1_PASSALL	0x02
#define MACON1_MARXEN	0x01

// ENC28J60 MACON2 Register Bit Definitions
#define MACON2_MARST	0x80
#define MACON2_RNDRST	0x40
#define MACON2_MARXRST	0x08
#define MACON2_RFUNRST	0x04
#define MACON2_MATXRST	0x02
#define MACON2_TFUNRST	0x01

// ENC28J60 MACON3 Register Bit Definitions
#define MACON3_PADCFG2	0x80
#define MACON3_PADCFG1	0x40
#define MACON3_PADCFG0	0x20
#define MACON3_TXCRCEN	0x10
#define MACON3_PHDRLEN	0x08
#define MACON3_HFRMLEN	0x04
#define MACON3_FRMLNEN	0x02
#define MACON3_FULDPX	0x01

// ENC28J60 MICMD Register Bit Definitions
#define MICMD_MIISCAN	0x02
#define MICMD_MIIRD		0x01

// ENC28J60 MISTAT Register Bit Definitions
#define MISTAT_NVALID	0x04
#define MISTAT_SCAN		0x02
#define MISTAT_BUSY		0x01

// ENC28J60 PHY PHCON1 Register Bit Definitions
#define	PHCON1_PRST		0x8000
#define	PHCON1_PLOOPBK	0x4000
#define	PHCON1_PPWRSV	0x0800
#define	PHCON1_PDPXMD	0x0100

// ENC28J60 PHY PHSTAT1 Register Bit Definitions
#define	PHSTAT1_PFDPX	0x1000
#define	PHSTAT1_PHDPX	0x0800
#define	PHSTAT1_LLSTAT	0x0004
#define	PHSTAT1_JBSTAT	0x0002

// ENC28J60 PHY PHCON2 Register Bit Definitions
#define PHCON2_FRCLINK	0x4000
#define PHCON2_TXDIS	0x2000
#define PHCON2_JABBER	0x0400
#define PHCON2_HDLDIS	0x0100

// ENC28J60 Packet Control Byte Bit Definitions
#define PKTCTRL_PHUGEEN		0x08
#define PKTCTRL_PPADEN		0x04
#define PKTCTRL_PCRCEN		0x02
#define PKTCTRL_POVERRIDE	0x01

// SPI operation codes
#define ENC28J60_READ_CTRL_REG	0x00
#define ENC28J60_READ_BUF_MEM	0x3A
#define ENC28J60_WRITE_CTRL_REG	0x40
#define ENC28J60_WRITE_BUF_MEM	0x7A
#define ENC28J60_BIT_FIELD_SET	0x80
#define ENC28J60_BIT_FIELD_CLR	0xA0
#define ENC28J60_SOFT_RESET		0xFF
